@@ -12,7 +12,7 @@
 // limitations under the License.
 
 use super::{
-    AckPolicy, Consumer, DeliverPolicy, FromConsumer, IntoConsumerConfig, ReplayPolicy,
+    backoff, AckPolicy, Consumer, DeliverPolicy, FromConsumer, IntoConsumerConfig, ReplayPolicy,
     StreamError, StreamErrorKind,
 };
 
@@ -34,7 +34,7 @@ use serde::{Deserialize, Serialize};
 use std::collections::HashMap;
 use std::task::{self, Poll};
 use std::{
-    io::{self, ErrorKind},
+    io::{self},
     pin::Pin,
     sync::Arc,
 };
@@ -293,8 +293,7 @@ pub struct Config {
 impl FromConsumer for Config {
     fn try_from_consumer_config(config: super::Config) -> Result<Self, crate::Error> {
         if config.deliver_subject.is_none() {
-            return Err(Box::new(io::Error::new(
-                ErrorKind::Other,
+            return Err(Box::new(io::Error::other(
                 "push consumer must have delivery subject",
             )));
         }
@@ -439,8 +438,7 @@ impl FromConsumer for OrderedConfig {
         Self: Sized,
     {
         if config.deliver_subject.is_none() {
-            return Err(Box::new(io::Error::new(
-                ErrorKind::Other,
+            return Err(Box::new(io::Error::other(
                 "push consumer must have delivery subject",
             )));
         }
@@ -549,8 +547,8 @@ impl Consumer<OrderedConfig> {
                             last_sequence.load(Ordering::Relaxed),
                         )
                     })
-                    .retries(5)
-                    .exponential_backoff(Duration::from_millis(500))
+                    .retries(u32::MAX)
+                    .custom_backoff(backoff)
                     .await;
                     if let Err(err) = consumer {
                         shutdown_tx.send(err).unwrap();
@@ -881,11 +879,9 @@ async fn recreate_ephemeral_consumer(
     stream_name: String,
     sequence: u64,
 ) -> Result<(), ConsumerRecreateError> {
-    let strategy =
-        tryhard::RetryFutureConfig::new(5).exponential_backoff(Duration::from_millis(500));
-
     let stream = tryhard::retry_fn(|| context.get_stream(stream_name.clone()))
-        .with_config(strategy)
+        .retries(u32::MAX)
+        .custom_backoff(backoff)
         .await
         .map_err(|err| {
             ConsumerRecreateError::with_source(ConsumerRecreateErrorKind::GetStream, err)
@@ -911,7 +907,8 @@ async fn recreate_ephemeral_consumer(
             }),
         )
     })
-    .with_config(strategy)
+    .retries(u32::MAX)
+    .custom_backoff(backoff)
     .await
     .map_err(|_| ConsumerRecreateError::new(ConsumerRecreateErrorKind::TimedOut))?
     .map_err(|err| ConsumerRecreateError::with_source(ConsumerRecreateErrorKind::Recreate, err))?;
