@@ -129,6 +129,9 @@ impl ObjectStore {
     {
         Box::pin(async move {
             let object_info = self.info(object_name).await?;
+            if object_info.deleted {
+                return Err(GetError::new(GetErrorKind::NotFound));
+            }
             if let Some(ref options) = object_info.options {
                 if let Some(link) = options.link.as_ref() {
                     if let Some(link_name) = link.name.as_ref() {
@@ -289,10 +292,7 @@ impl ObjectStore {
         let object_meta: ObjectMetadata = meta.into();
 
         // Fetch any existing object info, if there is any for later use.
-        let maybe_existing_object_info = match self.info(&object_meta.name).await {
-            Ok(object_info) => Some(object_info),
-            Err(_) => None,
-        };
+        let maybe_existing_object_info = (self.info(&object_meta.name).await).ok();
 
         let object_nuid = nuid::next();
         let chunk_subject = Subject::from(format!("$O.{}.C.{}", &self.name, &object_nuid));
@@ -937,13 +937,24 @@ pub struct Object {
     stream: crate::jetstream::stream::Stream,
 }
 
+impl std::fmt::Debug for Object {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        f.debug_struct("Object")
+            .field("info", &self.info)
+            .field("remaining_bytes", &self.remaining_bytes)
+            .field("has_pending_messages", &self.has_pending_messages)
+            .finish()
+    }
+}
+
 impl Object {
     pub(crate) fn new(info: ObjectInfo, stream: stream::Stream) -> Self {
+        let has_pending_messages = info.chunks > 0;
         Object {
             subscription: None,
             info,
             remaining_bytes: VecDeque::new(),
-            has_pending_messages: true,
+            has_pending_messages,
             digest: Some(Sha256::new()),
             subscription_future: None,
             stream,
@@ -1004,10 +1015,9 @@ impl tokio::io::AsyncRead for Object {
                     Poll::Ready(message) => match message {
                         Some(message) => {
                             let message = message.map_err(|err| {
-                                std::io::Error::new(
-                                    std::io::ErrorKind::Other,
-                                    format!("error from JetStream subscription: {err}"),
-                                )
+                                std::io::Error::other(format!(
+                                    "error from JetStream subscription: {err}"
+                                ))
                             })?;
                             let len = cmp::min(buf.remaining(), message.payload.len());
                             buf.put_slice(&message.payload[..len]);
@@ -1017,10 +1027,9 @@ impl tokio::io::AsyncRead for Object {
                             self.remaining_bytes.extend(&message.payload[len..]);
 
                             let info = message.info().map_err(|err| {
-                                std::io::Error::new(
-                                    std::io::ErrorKind::Other,
-                                    format!("error from JetStream subscription: {err}"),
-                                )
+                                std::io::Error::other(format!(
+                                    "error from JetStream subscription: {err}"
+                                ))
                             })?;
                             if info.pending == 0 {
                                 let digest = self.digest.take().map(Sha256::finish);
@@ -1051,8 +1060,7 @@ impl tokio::io::AsyncRead for Object {
                             }
                             Poll::Ready(Ok(()))
                         }
-                        None => Poll::Ready(Err(std::io::Error::new(
-                            std::io::ErrorKind::Other,
+                        None => Poll::Ready(Err(std::io::Error::other(
                             "subscription ended before reading whole object",
                         ))),
                     },
