@@ -17,7 +17,7 @@ use std::{
     collections::{self, HashMap},
     fmt::{self, Debug, Display},
     future::IntoFuture,
-    io::{self, ErrorKind},
+    io::{self},
     pin::Pin,
     str::FromStr,
     task::Poll,
@@ -37,8 +37,12 @@ use time::{serde::rfc3339, OffsetDateTime};
 
 use super::{
     consumer::{self, Consumer, FromConsumer, IntoConsumerConfig},
-    context::{RequestError, RequestErrorKind, StreamsError, StreamsErrorKind},
+    context::{
+        ConsumerInfoError, ConsumerInfoErrorKind, RequestError, RequestErrorKind, StreamsError,
+        StreamsErrorKind,
+    },
     errors::ErrorCode,
+    is_valid_name,
     message::{StreamMessage, StreamMessageError},
     response::Response,
     Context, Message,
@@ -603,7 +607,7 @@ impl<I> Stream<I> {
         .await
     }
 
-    /// Get a fist message from the stream for a given subject starting from provided sequence.
+    /// Get a first message from the stream for a given subject starting from provided sequence.
     /// This low-level API always reaches stream leader.
     /// This should be discouraged in favor of using [Stream::direct_get_first_for_subject].
     ///
@@ -958,17 +962,18 @@ impl<I> Stream<I> {
     pub async fn consumer_info<T: AsRef<str>>(
         &self,
         name: T,
-    ) -> Result<consumer::Info, crate::Error> {
+    ) -> Result<consumer::Info, ConsumerInfoError> {
         let name = name.as_ref();
 
+        if !is_valid_name(name) {
+            return Err(ConsumerInfoError::new(ConsumerInfoErrorKind::InvalidName));
+        }
+
         let subject = format!("CONSUMER.INFO.{}.{}", self.name, name);
 
         match self.context.request(subject, &json!({})).await? {
             Response::Ok(info) => Ok(info),
-            Response::Err { error } => Err(Box::new(std::io::Error::new(
-                ErrorKind::Other,
-                format!("nats: error while getting consumer info: {}", error),
-            ))),
+            Response::Err { error } => Err(error.into()),
         }
     }
 
@@ -1753,7 +1758,7 @@ impl TryFrom<RawMessage> for StreamMessage {
     fn try_from(value: RawMessage) -> Result<Self, Self::Error> {
         let decoded_payload = STANDARD
             .decode(value.payload)
-            .map_err(|err| Box::new(std::io::Error::new(ErrorKind::Other, err)))?;
+            .map_err(|err| Box::new(std::io::Error::other(err)))?;
         let decoded_headers = value
             .headers
             .map(|header| STANDARD.decode(header))
@@ -1787,18 +1792,14 @@ fn parse_headers(
     let mut lines = if let Ok(line) = std::str::from_utf8(buf) {
         line.lines().peekable()
     } else {
-        return Err(Box::new(std::io::Error::new(
-            ErrorKind::Other,
-            "invalid header",
-        )));
+        return Err(Box::new(std::io::Error::other("invalid header")));
     };
 
     if let Some(line) = lines.next() {
         let line = line
             .strip_prefix(HEADER_LINE)
             .ok_or_else(|| {
-                Box::new(std::io::Error::new(
-                    ErrorKind::Other,
+                Box::new(std::io::Error::other(
                     "version line does not start with NATS/1.0",
                 ))
             })?
@@ -1821,8 +1822,7 @@ fn parse_headers(
             }
         }
     } else {
-        return Err(Box::new(std::io::Error::new(
-            ErrorKind::Other,
+        return Err(Box::new(std::io::Error::other(
             "expected header information not found",
         )));
     };
@@ -1841,14 +1841,10 @@ fn parse_headers(
 
             headers.insert(
                 HeaderName::from_str(k)?,
-                HeaderValue::from_str(&s)
-                    .map_err(|err| Box::new(io::Error::new(ErrorKind::Other, err)))?,
+                HeaderValue::from_str(&s).map_err(|err| Box::new(io::Error::other(err)))?,
             );
         } else {
-            return Err(Box::new(std::io::Error::new(
-                ErrorKind::Other,
-                "malformed header line",
-            )));
+            return Err(Box::new(std::io::Error::other("malformed header line")));
         }
     }
 
